/**
 * 
 */

/**
 * @author nieminen
 *
 */
public class NeuralMath {

	
	/** 
	 * Computes A (1 x^T)^T 
	 * 
	 * @param res  will be re-assigned to the outcome
	 */
	public static void vecMatWithBias(double[] x, double[][] mat, double[] res) {
		int m = mat.length;
		int n = mat[0].length;
		
		for (int i=0; i<m; i++){
			res[i] = mat[i][0];
			for (int j=1; j<n; j++){
				res[i] += mat[i][j] * x[j-1];
			}
		}
	}

	/** Squashes the vector components with tanh. */
	public static void applyTansigActivation(double[] v) {
		for(int i=0;i<v.length;i++){
			v[i] = 2.0 / (1.0 + Math.exp(-2.0*v[i])) - 1.0;
		}		
	}

	/** Sets a = a - b. */
	public static void subtract(double[] a, double[] b) {
		for(int i=0;i<a.length;i++){
			a[i] -= b[i];
		}
	}

	/** The submatrix of W without bias column is transposed and it 
	 *  multiplies v; result is multiplied by the tanh derivatives of o
	 *  and left in o.
	 *
	 */
	public static void tanhDerivTimesSubmatTVec(double[][] W, double[] v, double[] o) {
		int m=W.length;
		int n=W[0].length;
		
		double dtanhj;
		
		for(int j=1;j<n;j++){
			
			dtanhj = (1.0 - o[j-1] * o[j-1]);
			o[j-1] = dtanhj * W[0][j]*v[0];
			
			for(int i=1;i<m;i++){
				o[j-1] += dtanhj * W[i][j]*v[i];
			}
		}
	}
	
	
	/** The submatrix of W without bias column is transposed and it 
	 *  multiplies v; result stored in res which is expected to pre-exist.
	 *
	 */
	public static void submatTVec(double[][] W, double[] v, double[] res) {
		int m=W.length;
		int n=W[0].length;
		
		for(int j=1;j<n;j++){
			res[j-1] = W[0][j]*v[0];
		}
		for(int i=1;i<m;i++){
			for(int j=1;j<n;j++){
				res[j-1] += W[i][j]*v[i];
			}
		}
	}

	public static void applyTansigDerivInPlace(double[] v) {
		  for(int i = 0; i<v.length; i++) {
		     v[i] = (1.0 - v[i] * v[i]);
		  }		
	}

	public static void diagTimesVec(double[] d, double[] y) {
		for (int i=1;i<d.length;i++){
			y[i] = d[i]*y[i];
		}
	}

	/** A = A + x (1 y)^T */
	public static void matPlusVecVecTwbias(double[][] A, double[] x, double[] y) {
		for(int i=0;i<A.length;i++){
			A[i][0] += x[i];
			for (int j=1;j<A[0].length;j++){
				A[i][j] += x[i] * y[j-1];
			}
		}
	}

	/** returns x^T y*/
	public static double dot(double[] x, double[] y) {
		double res = 0.0;
		for (int i=0;i<x.length;i++){
			res += x[i] * y[i];
		}
		return res;
	}

	/** Returns Sum (g_ijk)^2 over all ijk */
	public static double squareSumOfMatrices(double[][][] G) {
		double res = 0.0;
		for (double[][] each: G)
			for(double[] row: each)
				res += dot(row,row);
		return res;
	}

	/** Computes W = W + a G where W and G are collections of equally sized matrices */
	public static void addManyMatrices(double[][][] W, double a, double[][][] G) {
		for(int L=0;L<W.length; L++){
			for(int i=0;i<W[L].length;i++){
				for (int j=0;j<W[L][0].length;j++){
					W[L][i][j] += a * G[L][i][j];
				}
			}
		}		
	}

	public static void scaleManyMatrices(double[][][] matrices, double a) {
		for (double[][] mat: matrices)
			for(double[] row: mat)
				for(int j=0;j<row.length;j++)
					row[j] *= a;
	}
}
